@@ -10,11 +10,13 @@ import re
 
 modules = [
     "2chan",
+    "2chen",
     "35photo",
     "3dbooru",
     "420chan",
     "4chan",
     "500px",
+    "8chan",
     "8kun",
     "8muses",
     "adultempire",
@@ -55,6 +57,7 @@ modules = [
     "hentaihere",
     "hiperdex",
     "hitomi",
+    "hotleak",
     "idolcomplex",
     "imagebam",
     "imagechest",
@@ -89,6 +92,7 @@ modules = [
     "mememuseum",
     "myhentaigallery",
     "myportfolio",
+    "nana",
     "naver",
     "naverwebtoon",
     "newgrounds",
