@@ -34,13 +34,25 @@ class InstagramExtractor(Extractor):
     def __init__(self, match):
         Extractor.__init__(self, match)
         self.item = match.group(1)
+        self.api = None
         self.www_claim = "0"
         self.csrf_token = util.generate_token()
+        self._logged_in = True
         self._find_tags = re.compile(r"#\w+").findall
         self._cursor = None
 
     def items(self):
         self.login()
+
+        api = self.config("api")
+        if api is None or api == "auto":
+            api = InstagramRestAPI if self._logged_in else InstagramGraphqlAPI
+        elif api == "graphql":
+            api = InstagramGraphqlAPI
+        else:
+            api = InstagramRestAPI
+        self.api = api(self)
+
         data = self.metadata()
         videos = self.config("videos", True)
         previews = self.config("previews", False)
@@ -51,10 +63,11 @@ class InstagramExtractor(Extractor):
             if "__typename" in post:
                 post = self._parse_post_graphql(post)
             else:
-                post = self._parse_post_api(post)
+                post = self._parse_post_rest(post)
             post.update(data)
             files = post.pop("_files")
 
+            post["count"] = len(files)
             yield Message.Directory, post
             for file in files:
                 file.update(post)
@@ -107,63 +120,6 @@ class InstagramExtractor(Extractor):
 
         return response
 
-    def _request_api(self, endpoint, **kwargs):
-        url = "https://i.instagram.com/api" + endpoint
-        kwargs["headers"] = {
-            "X-CSRFToken"   : self.csrf_token,
-            "X-IG-App-ID"   : "936619743392459",
-            "X-IG-WWW-Claim": self.www_claim,
-        }
-        kwargs["cookies"] = {
-            "csrftoken": self.csrf_token,
-        }
-        return self.request(url, **kwargs).json()
-
-    def _request_graphql(self, query_hash, variables):
-        url = self.root + "/graphql/query/"
-        params = {
-            "query_hash": query_hash,
-            "variables" : json.dumps(variables),
-        }
-        headers = {
-            "X-CSRFToken"     : self.csrf_token,
-            "X-IG-App-ID"     : "936619743392459",
-            "X-IG-WWW-Claim"  : self.www_claim,
-            "X-Requested-With": "XMLHttpRequest",
-        }
-        cookies = {
-            "csrftoken": self.csrf_token,
-        }
-        return self.request(
-            url, params=params, headers=headers, cookies=cookies,
-        ).json()["data"]
-
-    @memcache(keyarg=1)
-    def _user_by_screen_name(self, screen_name):
-        url = "https://www.instagram.com/{}/?__a=1&__d=dis".format(
-            screen_name)
-        headers = {
-            "Referer": "https://www.instagram.com/{}/".format(screen_name),
-            "X-CSRFToken"     : self.csrf_token,
-            "X-IG-App-ID"     : "936619743392459",
-            "X-IG-WWW-Claim"  : self.www_claim,
-            "X-Requested-With": "XMLHttpRequest",
-        }
-        cookies = {
-            "csrftoken": self.csrf_token,
-        }
-        return self.request(
-            url, headers=headers, cookies=cookies).json()["graphql"]["user"]
-
-    def _uid_by_screen_name(self, screen_name):
-        if screen_name.startswith("id:"):
-            return screen_name[3:]
-        return self._user_by_screen_name(screen_name)["id"]
-
-    def _media_by_id(self, post_id):
-        endpoint = "/v1/media/{}/info/".format(post_id)
-        return self._pagination_api(endpoint)
-
     def login(self):
         self._username = None
         if not self._check_cookies(self.cookienames):
@@ -171,92 +127,13 @@ class InstagramExtractor(Extractor):
             if username:
                 self._username = username
                 self._update_cookies(_login_impl(self, username, password))
+            else:
+                self._logged_in = False
         self.session.cookies.set(
             "csrftoken", self.csrf_token, domain=self.cookiedomain)
 
-    def _parse_post_graphql(self, post):
-        typename = post["__typename"]
-
-        if post.get("is_video") and "video_url" not in post:
-            media = next(self._media_by_id(post["id"]))
-            return self._parse_post_api(media)
-
-        if typename == "GraphSidecar" and \
-                "edge_sidecar_to_children" not in post:
-            media = next(self._media_by_id(post["id"]))
-            return self._parse_post_api(media)
-
-        pinned = post.get("pinned_for_users", ())
-        if pinned:
-            for index, user in enumerate(pinned):
-                pinned[index] = int(user["id"])
-
-        owner = post["owner"]
-        data = {
-            "typename"   : typename,
-            "date"       : text.parse_timestamp(post["taken_at_timestamp"]),
-            "likes"      : post["edge_media_preview_like"]["count"],
-            "pinned"     : pinned,
-            "owner_id"   : owner["id"],
-            "username"   : owner.get("username"),
-            "fullname"   : owner.get("full_name"),
-            "post_id"    : post["id"],
-            "post_shortcode": post["shortcode"],
-            "post_url"   : "{}/p/{}/".format(self.root, post["shortcode"]),
-            "description": text.parse_unicode_escapes("\n".join(
-                edge["node"]["text"]
-                for edge in post["edge_media_to_caption"]["edges"]
-            )),
-        }
-
-        tags = self._find_tags(data["description"])
-        if tags:
-            data["tags"] = sorted(set(tags))
-
-        location = post.get("location")
-        if location:
-            data["location_id"] = location["id"]
-            data["location_slug"] = location["slug"]
-            data["location_url"] = "{}/explore/locations/{}/{}/".format(
-                self.root, location["id"], location["slug"])
-
-        data["_files"] = files = []
-        if "edge_sidecar_to_children" in post:
-            for num, edge in enumerate(
-                    post["edge_sidecar_to_children"]["edges"], 1):
-                node = edge["node"]
-                dimensions = node["dimensions"]
-                media = {
-                    "num": num,
-                    "media_id"   : node["id"],
-                    "shortcode"  : (node.get("shortcode") or
-                                    shortcode_from_id(node["id"])),
-                    "display_url": node["display_url"],
-                    "video_url"  : node.get("video_url"),
-                    "width"      : dimensions["width"],
-                    "height"     : dimensions["height"],
-                    "sidecar_media_id" : post["id"],
-                    "sidecar_shortcode": post["shortcode"],
-                }
-                self._extract_tagged_users(node, media)
-                files.append(media)
-        else:
-            dimensions = post["dimensions"]
-            media = {
-                "media_id"   : post["id"],
-                "shortcode"  : post["shortcode"],
-                "display_url": post["display_url"],
-                "video_url"  : post.get("video_url"),
-                "width"      : dimensions["width"],
-                "height"     : dimensions["height"],
-            }
-            self._extract_tagged_users(post, media)
-            files.append(media)
-
-        return data
-
-    def _parse_post_api(self, post):
-        if "items" in post:
+    def _parse_post_rest(self, post):
+        if "items" in post:  # story or highlight
             items = post["items"]
             reel_id = str(post["id"]).rpartition(":")[2]
             data = {
@@ -270,7 +147,7 @@ class InstagramExtractor(Extractor):
             if "created_at" in post:
                 data["date"] = text.parse_timestamp(post.get("created_at"))
 
-        else:
+        else:  # regular image/video post
             data = {
                 "post_id" : post["pk"],
                 "post_shortcode": post["code"],
@@ -344,6 +221,85 @@ class InstagramExtractor(Extractor):
 
         return data
 
+    def _parse_post_graphql(self, post):
+        typename = post["__typename"]
+
+        if self._logged_in:
+            if post.get("is_video") and "video_url" not in post:
+                post = self.api.media(post["id"])[0]
+            elif typename == "GraphSidecar" and \
+                    "edge_sidecar_to_children" not in post:
+                post = self.api.media(post["id"])[0]
+
+        pinned = post.get("pinned_for_users", ())
+        if pinned:
+            for index, user in enumerate(pinned):
+                pinned[index] = int(user["id"])
+
+        owner = post["owner"]
+        data = {
+            "typename"   : typename,
+            "date"       : text.parse_timestamp(post["taken_at_timestamp"]),
+            "likes"      : post["edge_media_preview_like"]["count"],
+            "pinned"     : pinned,
+            "owner_id"   : owner["id"],
+            "username"   : owner.get("username"),
+            "fullname"   : owner.get("full_name"),
+            "post_id"    : post["id"],
+            "post_shortcode": post["shortcode"],
+            "post_url"   : "{}/p/{}/".format(self.root, post["shortcode"]),
+            "description": text.parse_unicode_escapes("\n".join(
+                edge["node"]["text"]
+                for edge in post["edge_media_to_caption"]["edges"]
+            )),
+        }
+
+        tags = self._find_tags(data["description"])
+        if tags:
+            data["tags"] = sorted(set(tags))
+
+        location = post.get("location")
+        if location:
+            data["location_id"] = location["id"]
+            data["location_slug"] = location["slug"]
+            data["location_url"] = "{}/explore/locations/{}/{}/".format(
+                self.root, location["id"], location["slug"])
+
+        data["_files"] = files = []
+        if "edge_sidecar_to_children" in post:
+            for num, edge in enumerate(
+                    post["edge_sidecar_to_children"]["edges"], 1):
+                node = edge["node"]
+                dimensions = node["dimensions"]
+                media = {
+                    "num": num,
+                    "media_id"   : node["id"],
+                    "shortcode"  : (node.get("shortcode") or
+                                    shortcode_from_id(node["id"])),
+                    "display_url": node["display_url"],
+                    "video_url"  : node.get("video_url"),
+                    "width"      : dimensions["width"],
+                    "height"     : dimensions["height"],
+                    "sidecar_media_id" : post["id"],
+                    "sidecar_shortcode": post["shortcode"],
+                }
+                self._extract_tagged_users(node, media)
+                files.append(media)
+        else:
+            dimensions = post["dimensions"]
+            media = {
+                "media_id"   : post["id"],
+                "shortcode"  : post["shortcode"],
+                "display_url": post["display_url"],
+                "video_url"  : post.get("video_url"),
+                "width"      : dimensions["width"],
+                "height"     : dimensions["height"],
+            }
+            self._extract_tagged_users(post, media)
+            files.append(media)
+
+        return data
+
     @staticmethod
     def _extract_tagged_users(src, dest):
         dest["tagged_users"] = tagged_users = []
@@ -382,50 +338,13 @@ class InstagramExtractor(Extractor):
                                          "username" : user["username"],
                                          "full_name": user["full_name"]})
 
-    def _pagination_graphql(self, query_hash, variables):
-        cursor = self.config("cursor")
-        if cursor:
-            variables["after"] = cursor
+    def _init_cursor(self):
+        return self.config("cursor") or None
 
-        while True:
-            data = next(iter(self._request_graphql(
-                query_hash, variables)["user"].values()))
-
-            for edge in data["edges"]:
-                yield edge["node"]
-
-            info = data["page_info"]
-            if not info["has_next_page"]:
-                return
-            elif not data["edges"]:
-                s = "" if self.item.endswith("s") else "s"
-                raise exception.StopExtraction(
-                    "%s'%s posts are private", self.item, s)
-
-            variables["after"] = self._cursor = info["end_cursor"]
-            self.log.debug("Cursor: %s", self._cursor)
-
-    def _pagination_api(self, endpoint, params=None):
-        if params is None:
-            params = {}
-        while True:
-            data = self._request_api(endpoint, params=params)
-            yield from data["items"]
-
-            if not data["more_available"]:
-                return
-            params["max_id"] = data["next_max_id"]
-
-    def _pagination_api_post(self, endpoint, params, post=False):
-        while True:
-            data = self._request_api(endpoint, method="POST", data=params)
-            for item in data["items"]:
-                yield item["media"]
-
-            info = data["paging_info"]
-            if not info["more_available"]:
-                return
-            params["max_id"] = info["max_id"]
+    def _update_cursor(self, cursor):
+        self.log.debug("Cursor: %s", cursor)
+        self._cursor = cursor
+        return cursor
 
 
 class InstagramUserExtractor(InstagramExtractor):
@@ -446,13 +365,13 @@ class InstagramUserExtractor(InstagramExtractor):
             (InstagramHighlightsExtractor, base + "highlights/"),
             (InstagramPostsExtractor     , base + "posts/"),
             (InstagramReelsExtractor     , base + "reels/"),
-            (InstagramChannelExtractor   , base + "channel/"),
             (InstagramTaggedExtractor    , base + "tagged/"),
+            (InstagramChannelExtractor   , base + "channel/"),
         ), ("posts",))
 
 
 class InstagramPostsExtractor(InstagramExtractor):
-    """Extractor for ProfilePage posts"""
+    """Extractor for an Instagram user's posts"""
     subcategory = "posts"
     pattern = USER_PATTERN + r"/posts"
     test = ("https://www.instagram.com/instagram/posts/", {
@@ -461,13 +380,26 @@ class InstagramPostsExtractor(InstagramExtractor):
     })
 
     def posts(self):
-        query_hash = "69cba40317214236af40e7efa697781d"
-        variables = {"id": self._uid_by_screen_name(self.item), "first": 50}
-        return self._pagination_graphql(query_hash, variables)
+        uid = self.api.user_id(self.item)
+        return self.api.user_feed(uid)
+
+
+class InstagramReelsExtractor(InstagramExtractor):
+    """Extractor for an Instagram user's reels"""
+    subcategory = "reels"
+    pattern = USER_PATTERN + r"/reels"
+    test = ("https://www.instagram.com/instagram/reels/", {
+        "range": "40-60",
+        "count": ">= 20",
+    })
+
+    def posts(self):
+        uid = self.api.user_id(self.item)
+        return self.api.user_clips(uid)
 
 
 class InstagramTaggedExtractor(InstagramExtractor):
-    """Extractor for ProfilePage tagged posts"""
+    """Extractor for an Instagram user's tagged posts"""
     subcategory = "tagged"
     pattern = USER_PATTERN + r"/tagged"
     test = ("https://www.instagram.com/instagram/tagged/", {
@@ -485,8 +417,8 @@ class InstagramTaggedExtractor(InstagramExtractor):
             self.user_id = self.item[3:]
             return {"tagged_owner_id": self.user_id}
 
-        user = self._user_by_screen_name(self.item)
-        self.user_id = user["id"]
+        self.user_id = self.api.user_id(self.item)
+        user = self.api.user(self.item)
 
         return {
             "tagged_owner_id" : user["id"],
@@ -495,13 +427,11 @@ class InstagramTaggedExtractor(InstagramExtractor):
         }
 
     def posts(self):
-        endpoint = "/v1/usertags/{}/feed/".format(self.user_id)
-        params = {"count": 50}
-        return self._pagination_api(endpoint, params)
+        return self.api.user_tagged(self.user_id)
 
 
 class InstagramChannelExtractor(InstagramExtractor):
-    """Extractor for ProfilePage channel"""
+    """Extractor for an Instagram user's channel posts"""
     subcategory = "channel"
     pattern = USER_PATTERN + r"/channel"
     test = ("https://www.instagram.com/instagram/channel/", {
@@ -510,25 +440,25 @@ class InstagramChannelExtractor(InstagramExtractor):
     })
 
     def posts(self):
-        query_hash = "bc78b344a68ed16dd5d7f264681c4c76"
-        variables = {"id": self._uid_by_screen_name(self.item), "first": 50}
-        return self._pagination_graphql(query_hash, variables)
+        uid = self.api.user_id(self.item)
+        return self.api.user_clips(uid)
 
 
 class InstagramSavedExtractor(InstagramExtractor):
-    """Extractor for ProfilePage saved media"""
+    """Extractor for an Instagram user's saved media"""
     subcategory = "saved"
-    pattern = USER_PATTERN + r"/saved/?$"
-    test = ("https://www.instagram.com/instagram/saved/",)
+    pattern = USER_PATTERN + r"/saved(?:/all-posts)?/?$"
+    test = (
+        ("https://www.instagram.com/instagram/saved/"),
+        ("https://www.instagram.com/instagram/saved/all-posts/"),
+    )
 
     def posts(self):
-        query_hash = "2ce1d673055b99250e93b6f88f878fde"
-        variables = {"id": self._uid_by_screen_name(self.item), "first": 50}
-        return self._pagination_graphql(query_hash, variables)
+        return self.api.user_saved()
 
 
 class InstagramCollectionExtractor(InstagramExtractor):
-    """Extractor for ProfilePage saved collection media"""
+    """Extractor for Instagram collection"""
     subcategory = "collection"
     pattern = USER_PATTERN + r"/saved/([^/?#]+)/([^/?#]+)"
     test = (
@@ -546,13 +476,59 @@ class InstagramCollectionExtractor(InstagramExtractor):
         }
 
     def posts(self):
-        endpoint = "/v1/feed/collection/{}/posts/".format(self.collection_id)
-        for item in self._pagination_api(endpoint):
-            yield item["media"]
+        return self.api.user_collection(self.collection_id)
+
+
+class InstagramStoriesExtractor(InstagramExtractor):
+    """Extractor for Instagram stories"""
+    subcategory = "stories"
+    pattern = (r"(?:https?://)?(?:www\.)?instagram\.com"
+               r"/stories/(?:highlights/(\d+)|([^/?#]+)(?:/(\d+))?)")
+    test = (
+        ("https://www.instagram.com/stories/instagram/"),
+        ("https://www.instagram.com/stories/highlights/18042509488170095/"),
+        ("https://instagram.com/stories/geekmig/2724343156064789461"),
+    )
+
+    def __init__(self, match):
+        self.highlight_id, self.user, self.media_id = match.groups()
+        if self.highlight_id:
+            self.subcategory = InstagramHighlightsExtractor.subcategory
+        InstagramExtractor.__init__(self, match)
+
+    def posts(self):
+        if self.highlight_id:
+            reel_id = "highlight:" + self.highlight_id
+        else:
+            reel_id = self.api.user_id(self.user)
+
+        reels = self.api.reels_media(reel_id)
+
+        if self.media_id and reels:
+            reel = reels[0]
+            for item in reel["items"]:
+                if item["pk"] == self.media_id:
+                    reel["items"] = (item,)
+                    break
+            else:
+                raise exception.NotFoundError("story")
+
+        return reels
+
+
+class InstagramHighlightsExtractor(InstagramExtractor):
+    """Extractor for an Instagram user's story highlights"""
+    subcategory = "highlights"
+    pattern = USER_PATTERN + r"/highlights"
+    test = ("https://www.instagram.com/instagram/highlights",)
+
+    def posts(self):
+        uid = self.api.user_id(self.item)
+        return self.api.highlights_media(uid)
 
 
 class InstagramTagExtractor(InstagramExtractor):
-    """Extractor for TagPage"""
+    """Extractor for Instagram tags"""
     subcategory = "tag"
     directory_fmt = ("{category}", "{subcategory}", "{tag}")
     pattern = BASE_PATTERN + r"/explore/tags/([^/?#]+)"
@@ -565,27 +541,7 @@ class InstagramTagExtractor(InstagramExtractor):
         return {"tag": text.unquote(self.item)}
 
     def posts(self):
-        endpoint = "/v1/tags/{}/sections/".format(self.item)
-        data = {
-            "include_persistent": "0",
-            "max_id" : None,
-            "page"   : None,
-            "surface": "grid",
-            "tab"    : "recent",
-        }
-
-        while True:
-            info = self._request_api(endpoint, method="POST", data=data)
-
-            for section in info["sections"]:
-                for media in section["layout_content"]["medias"]:
-                    yield media["media"]
-
-            if not info.get("more_available"):
-                return
-
-            data["max_id"] = info["next_max_id"]
-            data["page"] = info["next_page"]
+        return self.api.tags_media(self.item)
 
 
 class InstagramPostExtractor(InstagramExtractor):
@@ -618,7 +574,6 @@ class InstagramPostExtractor(InstagramExtractor):
                 "width": int,
             }
         }),
-
         # GraphSidecar
         ("https://www.instagram.com/p/BoHk1haB5tM/", {
             "count": 5,
@@ -633,7 +588,6 @@ class InstagramPostExtractor(InstagramExtractor):
                 "username": "instagram",
             }
         }),
-
         # GraphVideo
         ("https://www.instagram.com/p/Bqxp0VSBgJg/", {
             "pattern": r"/46840863_726311431074534_7805566102611403091_n\.mp4",
@@ -651,7 +605,6 @@ class InstagramPostExtractor(InstagramExtractor):
                 "width": int,
             }
         }),
-
         # GraphVideo (IGTV)
         ("https://www.instagram.com/tv/BkQjCfsBIzi/", {
             "pattern": r"/10000000_597132547321814_702169244961988209_n\.mp4",
@@ -668,7 +621,6 @@ class InstagramPostExtractor(InstagramExtractor):
                 "width": int,
             }
         }),
-
         # GraphSidecar with 2 embedded GraphVideo objects
         ("https://www.instagram.com/p/BtOvDOfhvRr/", {
             "count": 2,
@@ -679,7 +631,6 @@ class InstagramPostExtractor(InstagramExtractor):
                 "video_url": str,
             }
         }),
-
         # GraphImage with tagged user
         ("https://www.instagram.com/p/B_2lf3qAd3y/", {
             "keyword": {
@@ -690,125 +641,326 @@ class InstagramPostExtractor(InstagramExtractor):
                 }]
             }
         }),
-
         # URL with username (#2085)
         ("https://www.instagram.com/dm/p/CW042g7B9CY/"),
-
         ("https://www.instagram.com/reel/CDg_6Y1pxWu/"),
     )
 
     def posts(self):
-        return self._media_by_id(id_from_shortcode(self.item))
+        return self.api.media(id_from_shortcode(self.item))
 
 
-class InstagramStoriesExtractor(InstagramExtractor):
-    """Extractor for Instagram stories"""
-    subcategory = "stories"
-    pattern = (r"(?:https?://)?(?:www\.)?instagram\.com"
-               r"/stories/(?:highlights/(\d+)|([^/?#]+)(?:/(\d+))?)")
-    test = (
-        ("https://www.instagram.com/stories/instagram/"),
-        ("https://www.instagram.com/stories/highlights/18042509488170095/"),
-        ("https://instagram.com/stories/geekmig/2724343156064789461"),
-    )
+class InstagramRestAPI():
 
-    def __init__(self, match):
-        self.highlight_id, self.user, self.media_id = match.groups()
-        if self.highlight_id:
-            self.subcategory = InstagramHighlightsExtractor.subcategory
-        InstagramExtractor.__init__(self, match)
+    def __init__(self, extractor):
+        self.extractor = extractor
 
-    def posts(self):
-        if self.highlight_id:
-            reel_id = "highlight:" + self.highlight_id
-        else:
-            reel_id = self._uid_by_screen_name(self.user)
-
-        endpoint = "/v1/feed/reels_media/"
-        params = {"reel_ids": reel_id}
-        reels = self._request_api(endpoint, params=params)["reels"]
-
-        if self.media_id:
-            reel = reels[reel_id]
-            for item in reel["items"]:
-                if item["pk"] == self.media_id:
-                    reel["items"] = (item,)
-                    break
-            else:
-                raise exception.NotFoundError("story")
-
-        return reels.values()
-
-
-class InstagramHighlightsExtractor(InstagramExtractor):
-    """Extractor for all Instagram story highlights of a user"""
-    subcategory = "highlights"
-    pattern = USER_PATTERN + r"/highlights"
-    test = ("https://www.instagram.com/instagram/highlights",)
-
-    def posts(self):
-        endpoint = "/v1/highlights/{}/highlights_tray/".format(
-            self._uid_by_screen_name(self.item))
-        tray = self._request_api(endpoint)["tray"]
-        reel_ids = [highlight["id"] for highlight in tray]
-
-        # Anything above 30 responds with statuscode 400.
-        # 30 can work, however, sometimes the API will respond with 560 or 500.
+    def highlights_media(self, user_id):
         chunk_size = 5
-        endpoint = "/v1/feed/reels_media/"
+        reel_ids = [hl["id"] for hl in self.highlights_tray(user_id)]
 
         for offset in range(0, len(reel_ids), chunk_size):
-            chunk_ids = reel_ids[offset : offset+chunk_size]
-            params = {"reel_ids": chunk_ids}
-            reels = self._request_api(endpoint, params=params)["reels"]
-            for reel_id in chunk_ids:
-                yield reels[reel_id]
+            yield from self.reels_media(
+                reel_ids[offset : offset+chunk_size])
 
+    def highlights_tray(self, user_id):
+        endpoint = "/v1/highlights/{}/highlights_tray/".format(user_id)
+        return self._call(endpoint)["tray"]
 
-class InstagramReelsExtractor(InstagramExtractor):
-    """Extractor for an Instagram user's reels"""
-    subcategory = "reels"
-    pattern = USER_PATTERN + r"/reels"
-    test = ("https://www.instagram.com/instagram/reels/", {
-        "range": "40-60",
-        "count": ">= 20",
-    })
+    def media(self, post_id):
+        endpoint = "/v1/media/{}/info/".format(post_id)
+        return self._pagination(endpoint)
 
-    def posts(self):
-        endpoint = "/v1/clips/user/"
+    def reels_media(self, reel_ids):
+        endpoint = "/v1/feed/reels_media/"
+        params = {"reel_ids": reel_ids}
+        return self._call(endpoint, params=params)["reels_media"]
+
+    def tags_media(self, tag):
+        for section in self.tags_sections(tag):
+            for media in section["layout_content"]["medias"]:
+                yield media["media"]
+
+    def tags_sections(self, tag):
+        endpoint = "/v1/tags/{}/sections/".format(tag)
         data = {
-            "target_user_id": self._uid_by_screen_name(self.item),
-            "page_size"     : "50",
+            "include_persistent": "0",
+            "max_id" : None,
+            "page"   : None,
+            "surface": "grid",
+            "tab"    : "recent",
         }
+        return self._pagination_sections(endpoint, data)
 
-        return self._pagination_api_post(endpoint, data)
+    @memcache(keyarg=1)
+    def user(self, screen_name):
+        endpoint = "/v1/users/web_profile_info/"
+        params = {"username": screen_name}
+        return self._call(endpoint, params=params)["data"]["user"]
+
+    def user_id(self, screen_name):
+        if screen_name.startswith("id:"):
+            return screen_name[3:]
+        user = self.user(screen_name)
+        if user is None:
+            raise exception.AuthorizationError(
+                "Login required to access this profile")
+        if user["is_private"] and not user["followed_by_viewer"]:
+            name = user["username"]
+            s = "" if name.endswith("s") else "s"
+            raise exception.StopExtraction("%s'%s posts are private", name, s)
+        return user["id"]
+
+    def user_clips(self, user_id):
+        endpoint = "/v1/clips/user/"
+        data = {"target_user_id": user_id, "page_size": "50"}
+        return self._pagination_post(endpoint, data)
+
+    def user_collection(self, collection_id):
+        endpoint = "/v1/feed/collection/{}/posts/".format(collection_id)
+        params = {"count": 50}
+        return self._pagination(endpoint, params, media=True)
+
+    def user_feed(self, user_id):
+        endpoint = "/v1/feed/user/{}/".format(user_id)
+        params = {"count": 30}
+        return self._pagination(endpoint, params)
+
+    def user_saved(self):
+        endpoint = "/v1/feed/saved/posts/"
+        params = {"count": 50}
+        return self._pagination(endpoint, params, media=True)
+
+    def user_tagged(self, user_id):
+        endpoint = "/v1/usertags/{}/feed/".format(user_id)
+        params = {"count": 50}
+        return self._pagination(endpoint, params)
+
+    def _call(self, endpoint, **kwargs):
+        extr = self.extractor
+
+        url = "https://i.instagram.com/api" + endpoint
+        kwargs["headers"] = {
+            "X-CSRFToken"     : extr.csrf_token,
+            "X-Instagram-AJAX": "1006242110",
+            "X-IG-App-ID"     : "936619743392459",
+            "X-ASBD-ID"       : "198387",
+            "X-IG-WWW-Claim"  : extr.www_claim,
+            "Origin"          : extr.root,
+            "Referer"         : extr.root + "/",
+        }
+        kwargs["cookies"] = {
+            "csrftoken": extr.csrf_token,
+        }
+        return extr.request(url, **kwargs).json()
+
+    def _pagination(self, endpoint, params=None, media=False):
+        if params is None:
+            params = {}
+        extr = self.extractor
+        params["max_id"] = extr._init_cursor()
+
+        while True:
+            data = self._call(endpoint, params=params)
+
+            if media:
+                for item in data["items"]:
+                    yield item["media"]
+            else:
+                yield from data["items"]
+
+            if not data.get("more_available"):
+                return
+            params["max_id"] = extr._update_cursor(data["next_max_id"])
+
+    def _pagination_post(self, endpoint, params):
+        extr = self.extractor
+        params["max_id"] = extr._init_cursor()
+
+        while True:
+            data = self._call(endpoint, method="POST", data=params)
+
+            for item in data["items"]:
+                yield item["media"]
+
+            info = data["paging_info"]
+            if not info.get("more_available"):
+                return
+            params["max_id"] = extr._update_cursor(info["max_id"])
+
+    def _pagination_sections(self, endpoint, params):
+        extr = self.extractor
+        params["max_id"] = extr._init_cursor()
+
+        while True:
+            info = self._call(endpoint, method="POST", data=params)
+
+            yield from info["sections"]
+
+            if not info.get("more_available"):
+                return
+            params["page"] = info["next_page"]
+            params["max_id"] = extr._update_cursor(info["next_max_id"])
 
 
-@cache(maxage=360*24*3600, keyarg=1)
+class InstagramGraphqlAPI():
+
+    def __init__(self, extractor):
+        self.extractor = extractor
+        self.user_collection = self.user_saved = self.reels_media = \
+            self.highlights_media = self._login_required
+        self._json_dumps = json.JSONEncoder(separators=(",", ":")).encode
+
+        api = InstagramRestAPI(extractor)
+        self.user = api.user
+        self.user_id = api.user_id
+
+    @staticmethod
+    def _login_required(_=None):
+        raise exception.AuthorizationError("Login required")
+
+    def highlights_tray(self, user_id):
+        query_hash = "d4d88dc1500312af6f937f7b804c68c3"
+        variables = {
+            "user_id": user_id,
+            "include_chaining": False,
+            "include_reel": False,
+            "include_suggested_users": False,
+            "include_logged_out_extras": True,
+            "include_highlight_reels": True,
+            "include_live_status": False,
+        }
+        edges = (self._call(query_hash, variables)["user"]
+                 ["edge_highlight_reels"]["edges"])
+        return [edge["node"] for edge in edges]
+
+    def media(self, post_id):
+        query_hash = "9f8827793ef34641b2fb195d4d41151c"
+        variables = {
+            "shortcode": shortcode_from_id(post_id),
+            "child_comment_count": 3,
+            "fetch_comment_count": 40,
+            "parent_comment_count": 24,
+            "has_threaded_comments": True,
+        }
+        media = self._call(query_hash, variables).get("shortcode_media")
+        return (media,) if media else ()
+
+    def tags_media(self, tag):
+        query_hash = "9b498c08113f1e09617a1703c22b2f32"
+        variables = {"tag_name": text.unescape(tag), "first": 50}
+        return self._pagination(query_hash, variables,
+                                "hashtag", "edge_hashtag_to_media")
+
+    def user_clips(self, user_id):
+        query_hash = "bc78b344a68ed16dd5d7f264681c4c76"
+        variables = {"id": user_id, "first": 50}
+        return self._pagination(query_hash, variables)
+
+    def user_feed(self, user_id):
+        query_hash = "69cba40317214236af40e7efa697781d"
+        variables = {"id": user_id, "first": 50}
+        return self._pagination(query_hash, variables)
+
+    def user_tagged(self, user_id):
+        query_hash = "be13233562af2d229b008d2976b998b5"
+        variables = {"id": user_id, "first": 50}
+        return self._pagination(query_hash, variables)
+
+    def _call(self, query_hash, variables):
+        extr = self.extractor
+
+        url = "https://www.instagram.com/graphql/query/"
+        params = {
+            "query_hash": query_hash,
+            "variables" : self._json_dumps(variables),
+        }
+        headers = {
+            "Accept"          : "*/*",
+            "X-CSRFToken"     : extr.csrf_token,
+            "X-Instagram-AJAX": "1006267176",
+            "X-IG-App-ID"     : "936619743392459",
+            "X-ASBD-ID"       : "198387",
+            "X-IG-WWW-Claim"  : extr.www_claim,
+            "X-Requested-With": "XMLHttpRequest",
+            "Referer"         : extr.root + "/",
+        }
+        cookies = {
+            "csrftoken": extr.csrf_token,
+        }
+        return extr.request(
+            url, params=params, headers=headers, cookies=cookies,
+        ).json()["data"]
+
+    def _pagination(self, query_hash, variables,
+                    key_data="user", key_edge=None):
+        extr = self.extractor
+        variables["after"] = extr._init_cursor()
+
+        while True:
+            data = self._call(query_hash, variables)[key_data]
+            data = data[key_edge] if key_edge else next(iter(data.values()))
+
+            for edge in data["edges"]:
+                yield edge["node"]
+
+            info = data["page_info"]
+            if not info["has_next_page"]:
+                return
+            elif not data["edges"]:
+                s = "" if self.item.endswith("s") else "s"
+                raise exception.StopExtraction(
+                    "%s'%s posts are private", self.item, s)
+
+            variables["after"] = extr._update_cursor(info["end_cursor"])
+
+
+@cache(maxage=90*24*3600, keyarg=1)
 def _login_impl(extr, username, password):
     extr.log.info("Logging in as %s", username)
 
-    url = extr.root + "/accounts/login/"
-    page = extr.request(url).text
+    user_agent = ("Mozilla/5.0 (Linux; Android 13) AppleWebKit/537.36 "
+                  "(KHTML, like Gecko) Chrome/106.0.5249.79 Mobile "
+                  "Safari/537.36 Instagram 255.1.0.17.102")
 
     headers = {
-        "X-Web-Device-Id" : text.extract(page, '"device_id":"', '"')[0],
+        "User-Agent"    : user_agent,
+        "Sec-Fetch-Dest": "document",
+        "Sec-Fetch-Mode": "navigate",
+        "Sec-Fetch-Site": "none",
+        "Sec-Fetch-User": "?1",
+    }
+    url = extr.root + "/accounts/login/"
+    response = extr.request(url, headers=headers)
+
+    extract = text.extract_from(response.text)
+    csrf_token = extract('"csrf_token":"', '"')
+    device_id = extract('"device_id":"', '"')
+    rollout_hash = extract('"rollout_hash":"', '"')
+
+    cset = extr.session.cookies.set
+    cset("csrftoken", csrf_token, domain=extr.cookiedomain)
+    cset("ig_did", device_id, domain=extr.cookiedomain)
+
+    headers = {
+        "User-Agent"      : user_agent,
+        "Accept"          : "*/*",
+        "X-CSRFToken"     : csrf_token,
+        "X-Instagram-AJAX": rollout_hash,
         "X-IG-App-ID"     : "936619743392459",
-        "X-ASBD-ID"       : "437806",
+        "X-ASBD-ID"       : "198387",
         "X-IG-WWW-Claim"  : "0",
         "X-Requested-With": "XMLHttpRequest",
+        "Origin"          : extr.root,
         "Referer"         : url,
+        "Sec-Fetch-Dest"  : "empty",
+        "Sec-Fetch-Mode"  : "cors",
+        "Sec-Fetch-Site"  : "same-origin",
     }
-    url = extr.root + "/data/shared_data/"
-    data = extr.request(url, headers=headers).json()
-
-    headers["X-CSRFToken"] = data["config"]["csrf_token"]
-    headers["X-Instagram-AJAX"] = data["rollout_hash"]
-    headers["Origin"] = extr.root
     data = {
-        "username"     : username,
-        "enc_password" : "#PWD_INSTAGRAM_BROWSER:0:{}:{}".format(
+        "enc_password"        : "#PWD_INSTAGRAM_BROWSER:0:{}:{}".format(
             int(time.time()), password),
+        "username"            : username,
         "queryParams"         : "{}",
         "optIntoOneTap"       : "false",
         "stopDeletionNonce"   : "",
@@ -820,11 +972,8 @@ def _login_impl(extr, username, password):
     if not response.json().get("authenticated"):
         raise exception.AuthenticationError()
 
-    cget = extr.session.cookies.get
-    return {
-        name: cget(name)
-        for name in ("sessionid", "mid", "ig_did")
-    }
+    return {cookie.name: cookie.value
+            for cookie in extr.session.cookies}
 
 
 def id_from_shortcode(shortcode):
