@@ -8,7 +8,6 @@
 
 import sys
 import json
-import time
 import errno
 import logging
 import functools
@@ -35,10 +34,13 @@ class Job():
         self.status = 0
         self.url_key = extr.config("url-metadata")
 
+        path_key = extr.config("path-metadata")
+        path_proxy = output.PathfmtProxy(self)
+
         self._logger_extra = {
             "job"      : self,
             "extractor": extr,
-            "path"     : output.PathfmtProxy(self),
+            "path"     : path_proxy,
             "keywords" : output.KwdictProxy(self),
         }
         extr.log = self._wrap_logger(extr.log)
@@ -58,6 +60,8 @@ class Job():
         kwdict = extr.config("keywords")
         if kwdict:
             self.kwdict.update(kwdict)
+        if path_key:
+            self.kwdict[path_key] = path_proxy
 
         # predicates
         self.pred_url = self._prepare_predicates("image", True)
@@ -69,9 +73,10 @@ class Job():
         log = extractor.log
         msg = None
 
-        sleep = util.build_duration_func(extractor.config("sleep-extractor"))
+        sleep = util.build_duration_func(
+            extractor.config("sleep-extractor"))
         if sleep:
-            time.sleep(sleep())
+            extractor.sleep(sleep(), "extractor")
 
         try:
             for msg in extractor:
@@ -233,7 +238,7 @@ class DownloadJob(Job):
             return
 
         if self.sleep:
-            time.sleep(self.sleep())
+            self.extractor.sleep(self.sleep(), "download")
 
         # download from URL
         if not self.download(url):
@@ -522,11 +527,11 @@ class SimulationJob(DownloadJob):
         if not kwdict["extension"]:
             kwdict["extension"] = "jpg"
         self.pathfmt.set_filename(kwdict)
-        self.out.skip(self.pathfmt.path)
         if self.sleep:
-            time.sleep(self.sleep())
+            self.extractor.sleep(self.sleep(), "download")
         if self.archive:
             self.archive.add(kwdict)
+        self.out.skip(self.pathfmt.path)
 
     def handle_directory(self, kwdict):
         if not self.pathfmt:
@@ -692,17 +697,18 @@ class DataJob(Job):
         self.ascii = config.get(("output",), "ascii", ensure_ascii)
 
         private = config.get(("output",), "private")
-        self.filter = util.identity if private else util.filter_dict
+        self.filter = dict.copy if private else util.filter_dict
 
     def run(self):
+        extractor = self.extractor
         sleep = util.build_duration_func(
-            self.extractor.config("sleep-extractor"))
+            extractor.config("sleep-extractor"))
         if sleep:
-            time.sleep(sleep())
+            extractor.sleep(sleep(), "extractor")
 
         # collect data
         try:
-            for msg in self.extractor:
+            for msg in extractor:
                 self.dispatch(msg)
         except exception.StopExtraction:
             pass
